// -*- C++ -*-
/*!
 * @file  StereoCameraCalibration.h
 * @brief Stereo Camera Calibration, refers to opencv/samples/cpp/stereo_calib.cpp
 * @date  $Date$
 * @author Hiroaki Yaguchi, JSK
 *
 * $Id$
 */

#ifndef STEREOCAMERACALIBRATION_H
#define STEREOCAMERACALIBRATION_H

#include <rtm/Manager.h>
#include <rtm/DataFlowComponentBase.h>
#include <rtm/CorbaPort.h>
#include <rtm/DataInPort.h>
#include <rtm/DataOutPort.h>
#include <rtm/idl/BasicDataTypeSkel.h>
#include <rtm/idl/ExtendedDataTypesSkel.h>
#include <rtm/idl/InterfaceDataTypesSkel.h>

// Service implementation headers
// <rtc-template block="service_impl_h">

// </rtc-template>

// Service Consumer stub headers
// <rtc-template block="consumer_stub_h">

// </rtc-template>


#include <iostream>
#include <vector>
#include <string>
#include <algorithm>

#include "opencv2/calib3d/calib3d.hpp"
#include "opencv2/highgui/highgui.hpp"
#include "opencv2/imgproc/imgproc.hpp"

#include "ImageTranslater.h"

using namespace std;
using namespace cv;
using namespace RTC;


/*!
 * @class StereoCameraCalibration
 * @brief Stereo Camera Calibration
 *
 */
class StereoCameraCalibration
  : public RTC::DataFlowComponentBase
{
 public:
  /*!
   * @brief constructor
   * @param manager Maneger Object
   */
  StereoCameraCalibration(RTC::Manager* manager);

  /*!
   * @brief destructor
   */
  ~StereoCameraCalibration();

  // <rtc-template block="public_attribute">
  
  // </rtc-template>

  // <rtc-template block="public_operation">
  
  // </rtc-template>

  /***
   *
   * The initialize action (on CREATED->ALIVE transition)
   * formaer rtc_init_entry() 
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
   virtual RTC::ReturnCode_t onInitialize();

  /***
   *
   * The finalize action (on ALIVE->END transition)
   * formaer rtc_exiting_entry()
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onFinalize();

  /***
   *
   * The startup action when ExecutionContext startup
   * former rtc_starting_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onStartup(RTC::UniqueId ec_id);

  /***
   *
   * The shutdown action when ExecutionContext stop
   * former rtc_stopping_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onShutdown(RTC::UniqueId ec_id);

  /***
   *
   * The activated action (Active state entry action)
   * former rtc_active_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
   virtual RTC::ReturnCode_t onActivated(RTC::UniqueId ec_id);

  /***
   *
   * The deactivated action (Active state exit action)
   * former rtc_active_exit()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
   virtual RTC::ReturnCode_t onDeactivated(RTC::UniqueId ec_id);

  /***
   *
   * The execution action that is invoked periodically
   * former rtc_active_do()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
   virtual RTC::ReturnCode_t onExecute(RTC::UniqueId ec_id);

  /***
   *
   * The aborting action when main logic error occurred.
   * former rtc_aborting_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onAborting(RTC::UniqueId ec_id);

  /***
   *
   * The error action in ERROR state
   * former rtc_error_do()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onError(RTC::UniqueId ec_id);

  /***
   *
   * The reset action that is invoked resetting
   * This is same but different the former rtc_init_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onReset(RTC::UniqueId ec_id);
  
  /***
   *
   * The state update action that is invoked after onExecute() action
   * no corresponding operation exists in OpenRTm-aist-0.2.0
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onStateUpdate(RTC::UniqueId ec_id);

  /***
   *
   * The action that is invoked when execution context's rate is changed
   * no corresponding operation exists in OpenRTm-aist-0.2.0
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onRateChanged(RTC::UniqueId ec_id);


 protected:
  // <rtc-template block="protected_attribute">
  
  // </rtc-template>

  // <rtc-template block="protected_operation">
  
  // </rtc-template>

  int m_board_width,m_board_height;
  float m_square_size,m_pixel_size;
  int m_min_image_count,m_stop_count;
  string m_continuous_mode;
  bool continuousp;

  // DataInPort declaration
  // <rtc-template block="inport_declare">
  CameraImage m_LeftImage;
  /*!
   */
  InPort<CameraImage> m_LeftImageIn;
  CameraImage m_RightImage;
  /*!
   */
  InPort<CameraImage> m_RightImageIn;
  
  // </rtc-template>


  // DataOutPort declaration
  // <rtc-template block="outport_declare">

  CameraImage m_SelectedLeftImage;
  /*!
   */
  OutPort<CameraImage> m_LeftImageOut;
  CameraImage m_SelectedRightImage;
  /*!
   */
  OutPort<CameraImage> m_RightImageOut;

  // </rtc-template>

  // CORBA Port declaration
  // <rtc-template block="corbaport_declare">
  
  // </rtc-template>

  // Service declaration
  // <rtc-template block="service_declare">
  
  // </rtc-template>

  // Consumer declaration
  // <rtc-template block="consumer_declare">
  
  // </rtc-template>

 private:
  // <rtc-template block="private_attribute">
  Mat cvleftimage,cvrightimage;
  Mat cvleftimageOut,cvrightimageOut;
  bool isnew_left,isnew_right;

  Size imageSize;
  Size boardSize;
  float squareSize;
  vector<vector<Point2f> > imagePoints[2];
  vector<vector<Point3f> > objectPoints;
  int imagecount,stopcount;

  vector<Point2f> prev_corners_left,prev_corners_right;
  Mat objectPointsMat,imagePointsMat;
  vector < vector <float> > boardPoses;

  // </rtc-template>

  // <rtc-template block="private_operation">

  /*!
   * @brief finding chessboard and drwaing result
   */
  bool findChessboard(Mat &img,Mat &cimg,vector<Point2f> &corners);

  /*!
   * @brief checking direction of found corners
   */
  bool checkDirection(vector<Point2f> &corners);

  /*!
   * @brief checking motion of calibration board from previous pose
   * @param prev
   * @param curr
   */
  bool checkMotion(vector<Point2f> &prev,vector<Point2f> &curr);

  /*!
   * @brief estimating calibration board pose from screen points
   * @param points
   * @param pose
   */
  void estimatePose(vector<Point2f> &points,vector<float> &pose);

  /*!
   * @brief checking motion of calibration board from previous pose
   * @param prev_pose
   * @param curr_pose
   */
  bool checkMotion3D(vector<float> &prev_pose,vector<float> &curr_pose);

  /*!
   * @brief main routine of calibration
   */
  bool doCalibration();

  // </rtc-template>

};


extern "C"
{
  DLL_EXPORT void StereoCameraCalibrationInit(RTC::Manager* manager);
};

#endif // STEREOCAMERACALIBRATION_H

